function [C,in]=hcfft(B,tflag,iflag,in)
%
% Computes the HCFFT its adjoint, inverse, and adjoint inverse
%
% [C,in]=hcfft(B,tflag,iflag,in)
%
% B    = [if iflag = 'noinv'] 
%        contains fourier coefficients of hyperbolic cross (vector or
%        tensor)
%        [if iflag = 'inv']
%        contains function values over sparse grid (vector or tensor)
% tflag= 'notransp'  => (i)hcfft
%        'transp'    => adjoint (i)hcfft
% iflag= 'noinv'     => (adjoint) hcfft
%        'inv'       => (adjoint) ihcfft
% in   = with hcfft_initialize(d,n) produced object or
%        simple vector [d,n] => hcfft calls hcfft_initialize(d,n)
%        optional, if B is a tensor
%
%
% Example:
%   
%   d=3; n=4; in=hcfft_initialize(d,n);
%
%   B=rand(in.s(d,n+1),1);            % hyperbolic cross of dimension 3 and level 4
%   B=hcfft_vec2hc(B,d,n,in);         % filled with real values (sparse tensor)
%
%   A=hcfft(B,'notransp','noinv',in); % computes values of trigonometric polynomial
%                                     % with fourier coefficients B over sparse
%                                     % grid
%
%   C=hcfft(A,'notransp','inv', in);  % iHCFFT computes fourier coefficients from
%                                     % function values A over sparse grid
%
%   sum(sum(sum(abs(double(C-B)))))   % should be near 0
%
% See also: hcdft, nhcdft, nhcfft
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft.m 1102 2010-07-08 14:29:25Z kaemmerer $

    % B tensor or vector
    try
        B.size;
        tensor=1;
    catch
        tensor=0;
    end
    
    if ~(strcmp(tflag,'transp') || strcmp(tflag,'notransp'))
        disp('problem with argument tflag');
        return;
    end
    
    % iflag to tflag:
    if strcmp(iflag,'inv')
        tflag=['i',tflag];
    elseif strcmp(iflag,'noinv')
    else
        disp('problem with argument iflag');
        return;
    end
   
    
    % hcfft_initialize in, if it's not there
    if tensor==0
        % maybe in is [d,n]
        switch length(in)
            case 1
                % ok, in is ready
                [d,n]=size(in.s);
                n=n-1;
            case 2
                d=in(1);
                n=in(2);
                in=hcfft_initialize(d,n);
            otherwise
                disp('problem with argument "in"');
                return;
        end
        b=B;
    else
        
        Nv=size(B);
        d=length(Nv);
        n=round(log2(Nv(1)));
        if(any(Nv~=2^n))
            disp('Size of input matrix (tensor) must be the same power of 2 in each dimension');
            return;
        end

        switch nargin
            case 3
                % in must be computed from B
                in=hcfft_initialize(d,n);
            case 4
                switch length(in)
                    case 1
                        % look, if B has the same size like in
                        [d_in,n_in]=size(in.s);
                        n_in=n_in-1;
                        if(n_in~=n || d_in~=d)
                            disp('in and tensor B are not compatible');
                            return;
                        end
                    case 2
                        if(d~=in(1) || n~=in(2))
                            disp('in and tensor B are not compatible');
                            return;
                        else
                            in=hcfft_initialize(d,n);
                        end
                    otherwise
                        disp('problem with argument "in"');
                        return;
                end
            otherwise
                disp('problem with number of arguments');
                return;
        end
 
        switch tflag
            case {'notransp','transp'}
                % B are fourier coefficients over hyperbolic cross
                [b,in]=hcfft_hc2vec(B,in);
            case {'inotransp','itransp'}
                % B are functionvalues over sparse grid
                [b,in]=hcfft_sg2vec(B,in);
            otherwise
                disp('problem with tflag');
                return;
        end
    end

    % call the real functions:
    
    switch tflag
        case 'notransp'
            [C,in]=hcfft_notransp(b,d,n,in);
            tensor=tensor+2;
        case 'transp'
            [C,in]=ahcfft(b,d,n,in);
            tensor=tensor+2;
        case 'inotransp'
            [C,in]=ihcfft(b,d,n,in);
            tensor=tensor+4;
        case 'itransp'
            [C,in]=aihcfft(b,d,n,in);
            tensor=tensor+4;
        otherwise
            disp('problem with tflag');
            return;
    end
    
    switch tensor
        case 3
            % we have hcfft or ahcfft with tensor => hcfft_vec2sg
            [C,in]=hcfft_vec2sg(C,d,n,in);
        case 5
            % we have ihcfft or aihcfft with tensor => hcfft_vec2hc
            [C,in]=hcfft_vec2hc(C,d,n,in);
    end

end

function [B, in] = hcfft_notransp(B, d, n, in)
%Computes function values on the sparse grid out of the Fourier coefficents
%on the hyperbolic cross. Input vector B contains the Fourier coefficients,
%output vector B the sparse grid.

%needs hcfft_index.m
%in (struct with some initial matrices) is an optional argument

    global irev;
    for o=1:n+1
        o1=1:2^(o-1);
        irev(2^(o-1)+1:2^o)=o1(dfp(1,2^(o-1)));
    end
    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in hcfft');
    end
    z=zeros(1,d);   %starting point
    B = invHierarchical(B, d, n, z, z, d, in);
    B = invNodal(B, d, n, z, z, d, in);

    clear irev;
end

function [B, in] = ihcfft(B, d, n, in)
%Computes Fourier coefficents on the hyperbolic cross. Input vector B
%contains the sparse grid, output vector B the Fourier coefficients.

%needs hcfft_index.m
%in (struct with some initial matrices) is an optional argument

    global irev;
    for o=1:n+1
        o1=1:2^(o-1);
        irev(2^(o-1)+1:2^o)=o1(dfp(1,2^(o-1)));
    end
    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in ihcfft');
    end
    z=zeros(1,d);   %starting point
    B = toHierarchical(B, d, n, z, z, d, in);
    B = toNodal(B, d, n, z, z, d, in);
    clear irev;

end

function [B, in] = ahcfft(B, d, n, in)
%Computes adjoint HCFFT out of the Fourier coefficents
%on the hyperbolic cross. Input vector B contains the Fourier coefficients,
%output vector B the sparse grid.

%needs hcfft_index.m
%in (struct with some initial matrices) is an optional argument

    global irev;
    for o=1:n+1
        o1=1:2^(o-1);
        irev(2^(o-1)+1:2^o)=o1(dfp(1,2^(o-1)));
    end
    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in hcfft');
    end
    z=zeros(1,d);   %starting point
    B = adjInvNodal(B, d, n, z, z, d, in);
    B = adjInvHierarchical(B, d, n, z, z, d, in);

    clear irev;
end

function [B, in] = aihcfft(B, d, n, in)
%Computes adjoint IHCFFT. Input vector B contains the sparse grid, output
%vector B the Fourier coefficients on the hyperbolic cross.

%needs hcfft_index.m
%in (struct with some initial matrices) is an optional argument

    global irev;
    for o=1:n+1
        o1=1:2^(o-1);
        irev(2^(o-1)+1:2^o)=o1(dfp(1,2^(o-1)));
    end
    if(nargin==3)
        in=hcfft_initialize(d,n);
        disp('initialization in ihcfft');
    end
    z=zeros(1,d);   %starting point
    B = adjToNodal(B, d, n, z, z, d, in);
    B = adjToHierarchical(B, d, n, z, z, d, in);
    clear irev;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by hcfft_notransp

function [ B ] = invHierarchical( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns hierarchical fourier coefficients

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = hira(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=invHierarchical(B, d-1, n, l1, q1, d-1, in);   %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);           %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=invHierarchical(B, 1, n, l2, q2, d, in);       %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);       %move forward in (d-1)-dimensional subgrid
        end
    end
end

function [ B ] = invNodal( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns coefficients in time space

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = dehi(B(IND), n-norml);
        B(IND) = myifft(B(IND), n-norml);
        B(IND) = B(icInd(IND, n-norml));
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=invNodal(B, d-1, n, l1, q1, d-1, in);  %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);       %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=invNodal(B, 1, n, l2, q2, d, in);      %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);   %move forward in (d-1)-dimensional subgrid
        end
    end
end

function A = myifft(A, n)
%returns iFFT of A using scaled coefficients in the order 0, 1, -1, 2, -2, 3, ...

    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    B(1)=A(1);
    B(2:2^(n-1)+1)=A(2:2:2^n);
    B(2^n:-1:2^(n-1)+2)=A(3:2:2^n-1); 
    A=ifft(B)*(2^n);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by ihcfft

function [ B ] = toHierarchical( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns hierarchical fourier coefficients

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = myfft(B(cInd(IND, n-norml)), n-norml);
        B(IND) = hira(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=toHierarchical(B, d-1, n, l1, q1, d-1, in);   %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);           %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=toHierarchical(B, 1, n, l2, q2, d, in);       %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);       %move forward in (d-1)-dimensional subgrid
        end
    end
end

function [ B ] = toNodal( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns nodal fourier coefficients

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = dehi(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=toNodal(B, d-1, n, l1, q1, d-1, in);       %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);       %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=toNodal(B, 1, n, l2, q2, d, in);           %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);   %move forward in (d-1)-dimensional subgrid
        end
    end
end

function A = myfft(A, n)
%returns FFT of A that is scaled with 2^n, coefficients in the order
%0, 1, -1, 2, -2, 3, ...

    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    B=fft(A)/(2^n);
    A(1)=B(1);
    A(2:2:2^n)=B(2:2^(n-1)+1);
    A(3:2:2^n-1)=B(2^n:-1:2^(n-1)+2);      
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by ahcfft

function [ B ] = adjInvNodal( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = B(cInd(IND, n-norml));
        B(IND) = myaifft(B(IND), n-norml);
        B(IND) = tDehi(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=adjInvNodal(B, d-1, n, l1, q1, d-1, in);  %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);       %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=adjInvNodal(B, 1, n, l2, q2, d, in);      %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);   %move forward in (d-1)-dimensional subgrid
        end
    end
end

function [ B ] = adjInvHierarchical( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = tHira(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=adjInvHierarchical(B, d-1, n, l1, q1, d-1, in);   %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);           %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=adjInvHierarchical(B, 1, n, l2, q2, d, in);       %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);       %move forward in (d-1)-dimensional subgrid
        end
    end
end

function A = myaifft(A, n)
%returns conjugated iFFT of A using scaled coefficients in the order 0, 1,
%-1, 2, -2, 3, ... and performs adjoint permutation afterwards

    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    B=fft(A);
    A(1)=B(1);
    A(2:2:2^n)=B(2:2^(n-1)+1);
    A(3:2:2^n-1)=B(2^n:-1:2^(n-1)+2);    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by aihcfft

function [ B ] = adjToNodal( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns nodal fourier coefficients

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = tDehi(B(IND), n-norml);
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=adjToNodal(B, d-1, n, l1, q1, d-1, in);       %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);       %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=adjToNodal(B, 1, n, l2, q2, d, in);           %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);   %move forward in (d-1)-dimensional subgrid
        end
    end
end

function [ B ] = adjToHierarchical( B, d, n, l, q, dir, in )
%d - dimension of subgrid
%n - highest level
%(l,q) - starting point of the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%returns hierarchical fourier coefficients

%With starting point (l,q), dimension d of the (sub-)grid and dir the subgrid
%is well-defined. If d=1, the subgrid is parallel to the dimension dir. If
%d>1 the subgrid is parallel to the first d dimensions 

    if(d == 1)
        norml=sum(l);
        IND = readm(n, l, q, dir, norml, in);
        B(IND) = tHira(B(IND), n-norml);
        B(IND) = myafft(B(IND), n-norml);
        B(IND) = B(icInd(IND, n-norml));
    else
        l1 = l;
        q1 = q;
        while(sum(l1)<n)
            B=adjToHierarchical(B, d-1, n, l1, q1, d-1, in);   %call unidirectional scheme with (d-1)-dimensional subgrid with starting point (l1,q1)
            [l1 q1] = nextP(1, n, l1, q1, d);           %move forward in 1D subgrid along dimension d
        end
        
        l2 = l;
        q2 = q;
        while(sum(l2)<n)
            B=adjToHierarchical(B, 1, n, l2, q2, d, in);       %call scheme with 1D subgrid with starting point (l2,q2) and parallel to dimension d
            [l2 q2] = nextP(d-1, n, l2, q2, d-1);       %move forward in (d-1)-dimensional subgrid
        end
    end
end

function A = myafft(A, n)
%returns adjoint FFT of A that is scaled with 2^n, coefficients in the order
%0, 1, -1, 2, -2, 3, ...

    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    B(1)=A(1);
    B(2:2^(n-1)+1)=A(2:2:2^n);
    B(2^n:-1:2^(n-1)+2)=A(3:2:2^n-1); 
    A=ifft(B);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by hcfft_notransp and ihcfft

function A = dehi(A, n)
    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    for l=1:n
        for k=2^(l-1):(2^l-1)
            A(2^l-k)=A(2^l-k)-A(k+1);
        end
    end
end

function A = hira(A, n)
    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    for l=n:-1:1
        for k=(2^l-1):-1:2^(l-1)
            A(2^l-k)=A(2^l-k)+A(k+1);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by ahcfft and aihcfft

function A = tDehi(A, n)
%transposed dehi
    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    for l=n:(-1):1
        for k=(2^l-1):(-1):2^(l-1)
            A(k+1)=A(k+1)-A(2^l-k);
        end
    end
end

function A = tHira(A, n)
%transposed hira
    if(length(A)~=2^n)
        fprintf('length of array erroneous\n');
    end
    for l=1:n
        for k=2^(l-1):(2^l-1)
            A(k+1)=A(k+1)+A(2^l-k);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by hcfft_notransp and aihcfft

function IND=icInd(IND, p)
    global irev;
    IND=IND(irev(2^p+1:2^(p+1)));
    for j=1:p
        Y=2^(j-1)+1:2^j;
        IND(Y)=IND(Y(irev(2^(j-1)+1:2^j)));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by ihcfft and ahcfft

function IND=cInd(IND, p)
    global irev;
    for j=1:p
        Y=2^(j-1)+1:2^j;
        IND(Y)=IND(Y(irev(2^(j-1)+1:2^j)));
    end
    IND=IND(irev(2^p+1:2^(p+1)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% needed by hcfft_notransp, ihcfft, ahcfft and aihcfft

function [ l, q ] = nextP(d, n, l, q, dir)
%d - dimension of subgrid
%(l,q) - any point in the subgrid
%dir - (only for d=1) subgrid is parallel to dimension dir

%Returns next point in d-dimensional subgrid. If d>1, the subgrid is
%well-defined (parallel to the first d dimensions and point (l,q) is
%contained). In this case, we have to change one of the first d dimensions
%of (l,q) to get to the next point. If d=1, the subgrid is well-defined as
%well (parallel to dimension dir and point p is contained). We have to change
%dimension dir to get to the next point.

    if(d==1)
        [l q] = nextP1(l, q, dir);
    else
        [l q] = nextPd(l, q, 1, dir, n);
    end
end

function [ l q ] = nextP1(l, q, dir)
%next point in 1d subgrid
    if(q(dir) < 2^l(dir)-1)
        q(dir)=q(dir)+2;
    else
        l(dir) = l(dir)+1;
        q(dir) = 1;
    end
end

function [ l, q ] = nextPd(l, q, d1, dir, n)
%    l1=l;
%    q1=q;
    [l q] = nextP1(l, q, d1);
    if((sum(l) == n)&&(d1<dir))
        l(d1) = 0;
        q(d1) = 0;
        [l q] = nextPd(l, q, d1+1, dir, n);
    end
end


function IND = readm(n, l, q, dir, lev, in) 
%reads matrix line in dimension dir, starting from point (l,q)
%lev = |l|
    d=length(l);
    if(l(dir)>0)
        fprintf('error while readm\n');
    end
    p=n-lev;
    IND=zeros(1,2^p);

    IND(1)=hcfft_index(l, q, d, lev, in);
    for i=1:p
        l(dir)=i;
        for j=1:2^(i-1)
            q(dir)=2*j-1;
            IND(j+2^(i-1))=hcfft_index(l, q, d, lev+i, in);
        end
    end
end

function V = dfp(b,N)
%bit reversal
    V = [b];
    p2 = N/2;
    while (p2 >= 1)
        V = [V , V + p2];
        p2 = p2/2;
    end
end

