function [B in] = hcfft_hc2vec(A, in)
%
% Converts a hc tensor in a hc vector in order of the indexing function
% A is a tensor or a sparse tensor. hcfft_hc2vec reads its entries on the 
% hyperbolic cross and writes them into vector B
%
% [B in]=hcfft_hc2vec(A,in)
%
% A     = hyperbolic cross (sparse) tensor
% in    = (struct with some initial matrices) is an optional argument
%
% Example:
%
%   in=hcfft_initialize(2,3);
%   h_vec=rand(in.s(2,4),1);        % fourier coefficients over hc as vector
%   h_ten=hcfft_vec2hc(h_vec,in);   % hc as sparse tensor (possibly easier handling)
%   h_ten(4,4)=0;                   % manipulate the fourier coefficients
%                                   % at (0,0)~(2^(n-1),2^(n-1))
%   h_vec1=hcfft_hc2vec(h_ten,in);  % hc as vector for computation
%
% See also: hcfft_coordhc, hcfft_vec2hc
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_hc2vec.m 1102 2010-07-08 14:29:25Z kaemmerer $

    x=size(A);
    d=length(x);
    n=round(log2(x(1)));
    if(any(x~=2^n))
        disp('length of any dimension must be equal and a power of 2');
        return;
    end
    if(nargin==1)
        in=hcfft_initialize(d,n);
        %disp('initialization in hcfft_hc2vec');
    end
    B=zeros(in.s(d, n+1),1);
    global HCFFT_A HCFFT_B;
    HCFFT_A=A;
    HCFFT_B=B;
    l=zeros(1,d);
    for k=0:n   
        numl(l, 1, d, k, n, in);
    end
    B=HCFFT_B;
    clear HCFFT_A HCFFT_B;
end

function v = sigma(q,d)
    for i=1:d
        if(mod(q(i),2)==0)
            v(i)=-q(i)/2;
        else
            v(i)=(q(i)+1)/2;
        end 
    end
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, q1, d1, d, l, n, k, in)
    global HCFFT_A HCFFT_B;
    for i=1:ceil(2^(l(d1)-1))
        q1(d1)=i;
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, q1, d1+1, d, l, n, k, in);
        else
            ind=hcfft_index(l, q, d, k, in);
            v=sigma(floor(2.^(l-1))+q1-1, d)+2^(n-1);
            HCFFT_B(ind)=HCFFT_A(v);
        end
    end            
end

