function [z,M]=lhcfft_search(d,n,type,M,T)
%
%
%   function [z,M]=lhcfft_search(d,n,type,M,T)
%
%   d:      spatial dimension of hyperbolic cross
%   n:      refinement of hyperbolic cross
%   type:   search strategy:
%               - 'global' :        global search
%               - 'random' :        random genrating vector search
%               - 'krobov' :        global search over korobov vectors
%               - 'randkorobov' :   random korobov vector search
%               - 'fixedkorobov' :  search minimal lattice size for
%                                   korobov vector with parameter 
%                                   3*(2^(n-1))
%   M:      if length(M)==1:
%               search for lattice with lattice size <= M
%           if M is vector and length(M)>1:
%               search for lattice with min(M) <= lattice size <= max(M)
%   T:      time to search for random and korobov random search
%           note, that there will be searched until the computaional time
%           exceeds the input
%
%   
%
%   z:  generating vector
%   M:  lattice size
%
%   See also: lhcfft, lhcfft_init
%
%   Copyright 2010 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
%   $Id: lhcfft_search.m 1102 2010-07-08 14:29:25Z kaemmerer $


    % to save the found lattices in lhcfft_init_data_new
    % set sflag to 1
    
    sflag=1;
    

    if(nargin==3)
        M_min=0; M_max=min(2^(d*n),intmax);
    else
        switch(length(M))
            case 1
                M_min=0; M_max=M;
            otherwise
                M_min=ceil(min(M)); M_max=floor(max(M));
        end
    end

    switch(type)
        case 'global'
            [z,M]=lattice_search_global(d,n,M_min,M_max);
        case 'random'
            [z,M]=lattice_search_random(d,n,M_min,M_max,T);
        case 'korobov'
            [z,M]=lattice_search_korobov(d,n,M_min,M_max);
        case 'randkorobov'
            [z,M]=lattice_search_randkorobov(d,n,M_min,M_max,T);
        case 'fixedkorobov'
            [z,M]=lattice_search_fixedkorobov(d,n,M_min,M_max);
    end


    if(sflag==1 && M~=0)
        
    %   if not saved until now or lattice smaller than the saved, save
    %   it to lattice_init_data struct Lattices

        save_lattice(d,n,M,z,type,M_min);
    end
    

end


function save_lattice(d,n,M,z,type,M_min)


    % evtl. nochmal testen, ob das lattice tatsächlich nonaliasing ist
    % Außerdem muss getestet sein, dass M_min und M_max bei global
    % bzw. korobov und fixedkorobov nicht zu sehr einschränken
    
    % global nur speichern, wenn M_min passt und noch kein Wert da ist
    % korobov analog
    % fixed_korobov analog
    % random nur speichern, wenn entweder noch keiner da, oder aber kleiner
    % randkorobov analog

    in=hcfft_initialize(d,n);
    tosave=0;
    switch(type)
        case {'global','korobov','fixedkorobov'}
            if(M_min<=min([in.s(d,n+1),2^(2*n-2)]))
                % result not influenced by M_min
                tosave=1;
            end
            
        case {'random','randkorobov'}
            tosave=1;
    end
    
    if(tosave==1)
        
        if(exist('lhcfft_init_data_modified.mat','file'))
            load lhcfft_init_data_modified;
        else
            load lhcfft_init_data;
        end
        
        if(isfield(Lattices,sprintf('d%d',d)) ...
                && isfield(Lattices.(sprintf('d%d',d)),type) ...
                && (length(Lattices.(sprintf('d%d',d)).(type).M)<n ...
                || Lattices.(sprintf('d%d',d)).(type).M(n)>M ...
                || Lattices.(sprintf('d%d',d)).(type).M(n)==0))
                % there exists no entry or an entry with bigger M,
                % so save new lattice
                Lattices.(sprintf('d%d',d)).(type).M(n)=M;
                Lattices.(sprintf('d%d',d)).(type).z(n,:)=z;
        elseif(isfield(Lattices,sprintf('d%d',d)) ...
                && ~isfield(Lattices.(sprintf('d%d',d)),type))
                % dimension exists, but the search method not
                MM(n)=M;
                zz(n,1:d)=z;
                temp=struct('M',MM,'z',zz);
                Lattices.(sprintf('d%d',d)).(type)=temp;
        elseif(~isfield(Lattices,sprintf('d%d',d)))
            MM(n)=M;
            zz(n,1:d)=z;
            temp=struct('M',MM,'z',zz);
            temp=struct(type,temp);
            Lattices.(sprintf('d%d',d))=temp;
        end
        save('lhcfft_init_data_modified','Lattices');
    end
end


function [z,Mstar]=lattice_search_global(d,n,M_min,M_max)

% TODO
% schauen obs vielleicht schon ein globales der Art gibt,
% dann brauchen wir nicht suchen, sonst suchen

    h=hcfft_coordhc(d,n);
    M=max([length(h),2^(2*n-2),M_min])-1;
    Mstar=0;
    while(Mstar==0 && M<M_max)
        M=M+1;
        ztest=1:d;
        while(ztest~=0)
            if(~has_double_val(mod(h*ztest',M)))
                z=ztest;
                Mstar=M; break;
            end
            ztest=globzplusone(ztest,M);
        end
    end
    if(Mstar==0)
        z=zeros(1,d);
        warning('lhcfft_search:randkorobov','There exists no nonaliasing lattice of size M in [%d,%d]!',M_min,M_max);
    end
    
    
end

function z1=globzplusone(z,M)
    z1=z;
    d=length(z);
    for j=1:d-1
        if z1(j)<z1(j+1)-1
            z1(j)=z1(j)+1;
            if(j>1)
                z1(1:j-1)=1:j-1;
            end
            break
        end
    end
    if(z==z1 & z(d)<M-1)
        z1(d)=z(d)+1;
        z1(1:d-1)=1:d-1;
    elseif(z==z1 & z(d)==M-1)
        z1=0;
    end
    
end

function b=has_double_val(v)

v=sort(v);

for j=1:length(v)-1
    if(v(j)==v(j+1))
        b=1;
        return;
    end
end

b=0;

end



function [z,Mstar]=lattice_search_random(d,n,M_min,M_max,T)


        tested_z(n)=0;
        h=hcfft_coordhc(d,n);
        act_start=tic;
        act_time=0;
        Mstar=min([2^(n*d),M_max+1,intmax]);
        Mlb=max([size(h,1),2^(2*n-2),M_min]);
        z=0;
        while(act_time<T)
      
            zr=randi(min([2^(n*d)-1,Mstar-1,intmax]),1,d);
            tested_z(n)=tested_z(n)+1;

            if(~has_double_val(h*zr'))
                
%               for M=Mlb:Mstar-1 % Problem mit oberer Schranke??
%               Umbau zur while Schleife:
                M=Mlb-1;
                while(M+1<Mstar)
                    M=M+1;
                    if(~has_double_val(mod(h*zr',M)))
                        Mstar=M;
                        z=zr; break;
                    end
                end
            end
            act_time=toc(act_start);
        end
        
        if(z==0)
            warning('lhcfft_search:random','No nonaliasing lattice of size M in [%d,%d] found!',Mlb,Mstar-1);
            Mstar=0;
            z=zeros(1,d);
        end
                    
  
end

function [z,Mstar]=lattice_search_korobov(d,n,M_min,M_max)

% schauen obs vielleicht schon ein globales korobov der Art gibt,
% dann brauchen wir nicht suchen, sonst suchen
 
        h=hcfft_coordhc(d,n);

        a=3*2^(n-2)-1;
        Mstar=min([2^(n*d),M_max,intmax]);
        Mlb=max([2^(2*n-2),size(h,1),M_min]);
        z=0;
        zaehler=0;
        while(a<=Mstar-3*2^(n-2)-1)
            a=a+1;
            zaehler=zaehler+1;
            M=Mlb;
            zr(1)=1;
            for j=2:d
                zr(j)=mod(zr(j-1)*a,M);
            end
            while(~(M>=(a+1)*2^(n-1) || mod(M+2^(n-1),a)>=2^n) || (has_double_val(mod(h*zr',M)) && M<Mstar+1))
                M=M+1;
                for j=2:d
                    zr(j)=mod(zr(j-1)*a,M);
                end
            end
            if((length(z)==1 && M<=Mstar) || M<Mstar)
                Mstar=M;
                z=zr;
            end
        end

        if(z==0)
            warning('lhcfft_search:korobov','No nonaliasing lattice of Korobov form of size M in [%d,%d] found!',Mlb,Mstar);
            Mstar=0;
            z=zeros(1,d);
        end

end



function [z,Mstar]=lattice_search_randkorobov(d,n,M_min,M_max,T)
 
        
        h=hcfft_coordhc(d,n);
        Mlb=max([2^(2*n-2),size(h,1),M_min]);
        Mub=min([2^(n*d),M_max,intmax]);
        Mstar=Mub;
        tested_z=0;
        act_start=tic;
        act_time=0;
        z=0;
        while(act_time<T)
       
            M=Mlb;
            a=randi([3*2^(n-2),Mstar-3*2^(n-2)],1,1);
            zr(1)=1;
            for j=2:d
                zr(j)=mod(zr(j-1)*a,M);
            end
            tested_z=tested_z+1;
            while(~(M>=(mod(a,M)+1)*2^(n-1) || mod(M+2^(n-1),mod(a,M))>=2^n) || (has_double_val(mod(h*zr',M)) && M<=Mstar))
                M=M+1;
                for j=2:d
                    zr(j)=mod(zr(j-1)*a,M);
                end
            end
            if((length(z)==1 && M<=Mstar) || M<Mstar)
                Mstar=M;
                z=zr;
            end
            act_time=toc(act_start);
        end
   
        if(z==0)
            warning('lhcfft_search:randkorobov','No nonaliasing lattice of Korobov form of size M in [%d,%d] found!',Mlb,Mstar);
            Mstar=0;
            z=zeros(1,d);
        end
end



function [z,M]=lattice_search_fixedkorobov(d,n,M_min,M_max)

        a=3*2^(n-2);
        h=hcfft_coordhc(d,n);
        Mlb=max([(1+a)*2^(n-1),size(h,1),M_min]);
        M=Mlb;
        Mub=min([2^(n*d),intmax,M_max]);
        z(1)=1;
        for j=2:d
            z(j)=mod(z(j-1)*a,M);
        end
        while(has_double_val(mod(h*z',M)) && M<=Mub)
            M=M+1;
            for j=2:d
                z(j)=mod(z(j-1)*a,M);
            end
        end
        
        if(M>Mub)
            warning('lhcfft_search:fixedkorobov','There exists no nonaliasing lattice with generating vector z(3*2^(n-2)) of Korobov form with lattice size [%d,%d]!',Mlb,Mub);
            M=0;
            z=zeros(1,d);
        end
end