function [C S in]=nhcfft(B,x,m,alpha,tflag,in)
% Computes NHCFFT and aNHCFFT in 2d and 3d
%
% [C in S] = nhcfft(B,x,m,alpha,tflag,in)
%   
% B    = [if tflag='notransp']
%        fourier coefficients over hyperbolic cross of dimension d and 
%        level n (vector or tensor)
%        [if tflag='transp' or 'ttransp']
%        function values f(x)
% x    = points of T^d, where function values should be computed
%        (tflag='notransp')
%        points of T^d, where f(x)=B (tflag='transp' or 'ttransp')
% m    = spline order
% alpha= oversampling factor (1,2,3, ...)
% tflag= 'transp' or 'notransp' or 'ttransp'(see below)
% in   = hcfft_initialize(d,n+alpha) or 
%        simple vector [d,n+alpha] => hcfft calls hcfft_initialize
%        optional for B tensor of fourier coefficients, if tflag='notransp'
%
%
% - tflag='notransp':
%   If B are fourier coefficients over the hyperbolic cross, B can be a
%   vector or a tensor. Then the result is a vector C and S. S is a
%   vector of spline coefficients of the interpolation step. C are the
%   values of x of trigonometric polynom over hyperbolic cross B.
%
%        length(B)
%   C(k)~  sum    B(j)*exp(2*pi*i*h(j)*x(k)), k=1:length(x)
%          j=1
%
%   h(j) in H_n^d is the index of fourier coefficient B(j)
%
%
% - tflag='transp' or 'ttransp':
%   B are the function values f(x(j))=B(j). Then
%
%          length(x)
%   C(j)~   sum     B(k)*exp(-2*pi*i*h(j)*x(k))
%           k=1
%
%   C is a tensor hyperbolic cross if tflag is 'ttransp'.
%   If tflag is 'transp' S is in vector form.
%
%
%
% Example:
%
%   d=2; n=6; alpha=2; m=4; 
%   in=hcfft_initialize(d,n+alpha);
%
%   B=rand(in.s(d,n+1),1);               % fourier coefficients
%   
%   x=rand(100,d);
%
%   C=nhcfft(B,x,m,alpha,'notransp',in); % function values C(j)=f(x(j))
%
% See also: nhcdft, hcfft, hcdft
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcfft.m 1102 2010-07-08 14:29:25Z kaemmerer $

    try
        Nv=B.size;
        d=length(Nv);
        n=round(log2(Nv(1)));
        if(any(Nv~=2^n))
            disp('Size of input matrix must be the same power of 2 in each dimension');
            return;
        end
        tensor=1;
    catch
        tensor=0;
    end
        
    % test and initialize in
    if (tensor==1 && nargin==5) || (tensor==1 && length(in)==2 && d==in(1) && n+alpha==in(2))
        in=hcfft_initialize(d,n+alpha);
        B=hcfft_hc2vec(B,in);
    elseif tensor==1 && length(in)==1 && isequal(size(in.s),[d,n+alpha+1])
        B=hcfft_hc2vec(B,in);
    elseif tensor==0 && length(in)==2
        d=in(1); n=in(2)-alpha;
        in=hcfft_initialize(d,n+alpha);
    elseif tensor==0 && length(in)==1
        [d,n]=size(in.s);
        n=n-alpha-1;
    else
        disp('problem with in');
        return;
    end
 

    tflag=[tflag,num2str(d)];
    
    switch tflag
        case 'notransp2'
            [C S in]=nhcfft_2d(B,n,x,m,alpha,in);
        case 'notransp3'
            [C S in]=nhcfft_3d(B,n,x,m,alpha,in);
        case 'transp2'
            [C S in] = anhcfft_2d(B, n, x, m, alpha, in);            
        case 'transp3'
            [C S in] = anhcfft_3d(B, n, x, m, alpha, in);            
        case 'ttransp2'
            [C S in] = anhcfft_2d(B, n, x, m, alpha, in);
            C=hcfft_vec2hc(C,d,n);
        case 'ttransp3'
            [C S in] = anhcfft_3d(B, n, x, m, alpha, in);
            C=hcfft_vec2hc(C,d,n);            
        otherwise
            disp('problem with tflag or dimension');
            return;
    end

end


function [C S in] = nhcfft_2d(B, n, x, m, alpha, in)
%Approximates function values of trigonmetric polynomial given by hyperbolic
%cross in vector B of dimension 2 and level n at the points of vector x by
%interpolating with splines on values of HCFFT(B). size(x)=[M 2] i.e. each
%row of x contains a point at which the polynomial will be evaluated.
%Spline degree is m (possible values 2, 4, 6, 8, 10, 12, 14, 16),
%oversampling factor is alpha (1, 2, 3, ...).

%C contains the approximation of the trigonometric polynomial at the points
%in x and S contains the spline coefficients. With them, further
%evaluations can be done with 
%     bool_val_2d(S, m, x)

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=2)
        disp('The M points at which the trigonometric polynomial will be evaluated must be in a matrix of the size M x 2.')
        return;
    end
    if(nargin==5)
        in=hcfft_initialize(2,n+alpha);
        disp('initialization in nhcfft_2d');
    end
    HC=zeros(1, in.s(2,n+alpha+1));
    HC(1:in.s(2,n+1))=B;
    
    A1=hcfft(HC, 'notransp','noinv', in);
    S=nhcfft_bool_int(A1,m,'notransp',in);
    C=nhcfft_bool_val(S, m, x, 'notransp', n+alpha);
    b=size(B);
    if(b(2)>1)
        C=C.';
    end
end


function [C S in] = nhcfft_3d(B, n, x, m, alpha, in)
%Approximates function values of trigonmetric polynomial given by
%hyperbolic cross in vector B of dimension 3 and level n at the points of
%vector x by interpolating with splines on values of HCFFT(B). size(x)=[M
%3] i.e. each row of x contains a point at which the polynomial will be
%evaluated. Spline degree is m (possible values 2, 4, 6, 8, 10, 12, 14,
%16), oversampling factor is alpha (1, 2, 3, ...).

%C contains the approximation of the trigonometric polynomial at the points
%in x and S contains the spline coefficients. With them, further
%evaluations can be done with 
%     bool_val_3d(S, m, x)

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=3)
        disp('The M points at which the trigonometric polynomial will be evaluated must be in a matrix of the size M x 3.')
        return;
    end
    if(nargin==5)
        in=hcfft_initialize(3,n+alpha);
        disp('initialization in nhcfft_3d');
    end
    HC=zeros(1, in.s(3,n+alpha+1));
    HC(1:in.s(3,n+1))=B;
    
    A1=hcfft(HC, 'notransp','noinv', in);
    S=nhcfft_bool_int(A1,m,'notransp',in);
    C=nhcfft_bool_val(S, m, x, 'notransp',n+alpha);
    b=size(B);
    if(b(2)>1)
        C=C.';
    end
end


function [B a in] = anhcfft_2d(S, n, x, m, alpha, in)
%Computes adjoint NHCFFT in 2d, i.e. input are the function values in S
%(size M) at the points in x (size(x)=[M 2]). Output is a hyperbolic cross
%of level n. 
%Spline degree is m (possible values 2, 4, 6, 8, 10, 12, 14, 16),
%oversampling factor is alpha (1, 2, 3, ...).

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=2)
        disp('The M points at which the trigonometric polynomial is evaluated must be in a matrix of the size M x 2.')
        return;
    end
    M=s(1);
    if(length(S)~=M)
        disp('The vector containing the function S and the vector containing the points x must have the same length.');
        return;
    end
    if(nargin==5)
        in=hcfft_initialize(2,n+alpha);
        disp('initialization in anhcfft');
    end

    a=nhcfft_bool_val(S,m,x,'transp',n+alpha);
    T=nhcfft_bool_int(a,m,'transp',in);
    A=hcfft(T, 'transp','noinv', in);
    B=A(1:in.s(2,n+1));
    b=size(S);
    if(b(1)>1)
        B=B.';
    end
end

function [B a in] = anhcfft_3d(S, n, x, m, alpha, in)
%Computes adjoint NHCFFT in 3d, i.e. input are the function values in S
%(size M) at the points in x (size(x)=[M 3]). Output is a hyperbolic cross
%of level n. 
%Spline degree is m (possible values 2, 4, 6, 8, 10, 12, 14, 16),
%oversampling factor is alpha (1, 2, 3, ...).

%in (struct with some initial matrices) is an optional argument

    s=size(x);
    if(s(2)~=3)
        disp('The M points at which the trigonometric polynomial is evaluated must be in a matrix of the size M x 3.')
        return;
    end
    M=s(1);
    if(length(S)~=M)
        disp('The vector containing the function S and the vector containing the points x must have the same length.');
        return;
    end
    if(nargin==5)
        in=hcfft_initialize(3,n+alpha);
        disp('initialization in anhcfft');
    end
    
    a=nhcfft_bool_val(S,m,x,'transp',n+alpha);
    T=nhcfft_bool_int(a,m,'transp',in);
    A=hcfft(T, 'transp','noinv', in);
    B=A(1:in.s(3,n+1));
    b=size(S);
    if(b(1)>1)
        B=B.';
    end
end
