function [y]=nhcfft_bool_val(a,m,x,tflag,n)
%
% Computes interpolated function values at any points x in 1d, 2d or 3d.
% x is a column list with the points [x1; y1; ...] for 1d, 
% [x1 y1; x2 y2; x3 y3; ...] for 2d or 
% [x1 y1 z1; x2 y2 z2; x3 y3 z3; x4 y4 z4; ...] for 3d.
% If the x are only grid points, there would be a faster possibility however.
%
% [y]=nhcfft_bool_val(a,m,x,tflag,n)
%
% a     = coefficients of spline
% m     = order of spline (4,6,8,10,12,14,16)
% tflag = 'notransp' computes function values
%         'transp' adjoint computation
% n     = level (size) of hyperbolic cross, 
%         only needed for tflag='transp' and d=2 or d=3
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcfft_bool_val.m 1102 2010-07-08 14:29:25Z kaemmerer $

    sx=size(x);

    if sum(ismember([2,4,6,8,10,12,14,16],m))==0
        disp('Order of spline has to be 2, 4, 6, 8, 10, 12, 14 or 16.');
        return;
    end

    switch tflag

        case 'notransp'
            switch sx(2)
                case 1
                    y=spline_val(a,m,x);
                case 2
                    y=bool_val_2d(a,m,x);
                case 3
                    y=bool_val_3d(a,m,x);
                otherwise
                    disp('list of points must be column-shaped [x1 y1 (z1); x2 y2 (z2); ...]');
            end
        
        case 'transp'
            switch sx(2)
                case 1
                    y=aspline_val(a,m,x);
                case 2
                    y=abool_val_2d(a,m,x,n);
                case 3
                    y=abool_val_3d(a,m,x,n);
                otherwise
                    disp('adjoint algorithm only available for d=2 and d=3');
            end
            
        otherwise
            disp('Only "transp" or "notransp" is valid use for tflag');
    end
    
end

function [ y ] = bool_val_2d( a, m, x )
%Computes interpolated function values at any points x in 2d. x is a column
%list with the points [x1 y1; x2 y2; x3 y3; ...].
%If the x are only grid points, there would be a faster possibility however.
%m=order of spline (4,6,8 or 10), a=coefficients of spline

    %r = m/2;
    dj=length(a);
    k=0;
    while((k+1)*2^k<dj)
        k=k+1;
    end
    n=k;
    if(dj~=(n+1)*2^n)
        disp('length of input vector must be like (n+1)*2^n');
        return;
    end
    [xa x12]=size(x);
    if(x12~=2)
        disp('list of points must be column-shaped [x1 y1; x2 y2; ...]');
        return;
    end
    y=zeros(xa,1);
    x1=x(:,1);
    x2=x(:,2);

    x1=x1-floor(x1);   % now x1 is definitely in [0,1)
    x2=x2-floor(x2);   % now x2 is definitely in [0,1)
 
    for r=0:n
        z1=x1*2^r;
        z1f=floor(z1);
        z2=x2*2^(n-r);
        z2f=floor(z2);
        switch m
            case 2
                phi1=q2(z1-z1f);
                phi2=q2(z2-z2f);
            case 4
                phi1=q4(z1-z1f);
                phi2=q4(z2-z2f);
            case 6
                phi1=q6(z1-z1f);
                phi2=q6(z2-z2f);
            case 8
                phi1=q8(z1-z1f);
                phi2=q8(z2-z2f);
            case 10
                phi1=q10(z1-z1f);
                phi2=q10(z2-z2f);
            case 12
                phi1=q12(z1-z1f);
                phi2=q12(z2-z2f);
            case 14
                phi1=q14(z1-z1f);
                phi2=q14(z2-z2f);
            case 16
                phi1=q16(z1-z1f);
                phi2=q16(z2-z2f);
        end
        b=zeros(m,xa);
        for l=0:m-1
            for i=1:xa
                i2=mod(z2f(i)-l,2^(n-r));
                for k=0:m-1
                    ind=isp(n,r,mod(z1f(i)-k,2^r),i2);
                    b(l+1,i)=b(l+1,i)+phi1(k+1,i)*a(ind);
                end
            end
        end
        for i=1:xa
            y(i)=y(i)+phi2(:,i)'*b(:,i);
        end
    end
    
end


function [ s ] = bool_val_3d( a, m, x )
%Computes interpolated function values at any points x in 3d. x is a column
%list with the points [x1 y1 z1; x2 y2 z2; x3 y3 z3; ...].
%If the x are only grid points, there would be a faster possibility however.
%m=order of spline (2,4,6,8 or 10), a=coefficients of spline

%    r = m/2;
    dj=length(a);
    k=0;
    while((k+1)*(k+2)*2^(k-1)<dj)
        k=k+1;
    end
    n=k;
    if(dj~=(n+1)*(n+2)*2^(n-1))
        disp('length of input vector must be like (n+1)*(n+2)*2^(n-1)');
        return;
    end
    [xa x12]=size(x);
    if(x12~=3)
        disp('list of points must be column-shaped [x1 y1 z1; x2 y2 z2; ...]');
        return;
    end
    s=zeros(xa,1);
    u=zeros(m,1);
    t=u;
    phix=zeros(n+1,m);
    phiy=phix;
    phiz=phix;

    x=x-floor(x);   % now x is definitely in [0,1)
 
    for i=1:xa
        for j=0:n
            xx=x(i,:)*2^j;
            xxf=floor(xx);
            switch m
                case 2
                    phix(j+1,:)=q2(xx(1)-xxf(1));
                    phiy(j+1,:)=q2(xx(2)-xxf(2));
                    phiz(j+1,:)=q2(xx(3)-xxf(3));
                case 4
                    phix(j+1,:)=q4(xx(1)-xxf(1));
                    phiy(j+1,:)=q4(xx(2)-xxf(2));
                    phiz(j+1,:)=q4(xx(3)-xxf(3));
                case 6
                    phix(j+1,:)=q6(xx(1)-xxf(1));
                    phiy(j+1,:)=q6(xx(2)-xxf(2));
                    phiz(j+1,:)=q6(xx(3)-xxf(3));
                case 8
                    phix(j+1,:)=q8(xx(1)-xxf(1));
                    phiy(j+1,:)=q8(xx(2)-xxf(2));
                    phiz(j+1,:)=q8(xx(3)-xxf(3));
                case 10
                    phix(j+1,:)=q10(xx(1)-xxf(1));
                    phiy(j+1,:)=q10(xx(2)-xxf(2));
                    phiz(j+1,:)=q10(xx(3)-xxf(3));
                case 12
                    phix(j+1,:)=q12(xx(1)-xxf(1));
                    phiy(j+1,:)=q12(xx(2)-xxf(2));
                    phiz(j+1,:)=q12(xx(3)-xxf(3));
                case 14
                    phix(j+1,:)=q14(xx(1)-xxf(1));
                    phiy(j+1,:)=q14(xx(2)-xxf(2));
                    phiz(j+1,:)=q14(xx(3)-xxf(3));
                case 16
                    phix(j+1,:)=q16(xx(1)-xxf(1));
                    phiy(j+1,:)=q16(xx(2)-xxf(2));
                    phiz(j+1,:)=q16(xx(3)-xxf(3));
            end
        end
        for j=0:n
            for k3=0:m-1
                u(k3+1)=0;
                k3t=mod(floor(2^(n-j)*x(i,3))-k3, 2^(n-j));
                for j1=0:j
                    for k2=0:m-1
                        t(k2+1)=0;
                        k2t=mod(floor(2^(j-j1)*x(i,2))-k2, 2^(j-j1));
                        for k1=0:m-1
                            k1t=mod(floor(2^j1*x(i,1))-k1, 2^j1);
                            t(k2+1)=t(k2+1)+a(isp3(n, j, j1, k1t, k2t, k3t))*phix(j1+1, k1+1);
                        end
                    end
                    u(k3+1)=u(k3+1)+phiy(j-j1+1,:)*t;
                end
            end
            s(i)=s(i)+phiz(n-j+1,:)*u;
        end
    end
end


function [ a ] = abool_val_2d( y, m, x, n )
%Computes adjoint spline coefficients at any points x in 2d from function
%values y. x is a column list with the points [x1 y1; x2 y2; x3 y3; ...].
%If the x are only grid points, there would be a faster possibility however.
%m=order of spline (4,6,8 or 10), n=level, a=coefficients of spline

    r = m/2;
    if((r~=1) && (r~=2) && (r~=3) && (r~=4) && (r~=5) && (r~=6) && (r~=7) && (r~=8))
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    [xa x12]=size(x);
    if(x12~=2)
        disp('list of points must be column-shaped [x1 y1; x2 y2; ...]');
        return;
    end
    a=zeros((n+1)*2^n,1);
    x1=x(:,1);
    x2=x(:,2);

    x1=x1-floor(x1);   % now x1 is definitely in [0,1)
    x2=x2-floor(x2);   % now x2 is definitely in [0,1)
 
    for r=0:n
        z1=x1*2^r;
        z1f=floor(z1);
        z2=x2*2^(n-r);
        z2f=floor(z2);
        switch m
            case 2
                phi1=q2(z1-z1f);
                phi2=q2(z2-z2f);
            case 4
                phi1=q4(z1-z1f);
                phi2=q4(z2-z2f);
            case 6
                phi1=q6(z1-z1f);
                phi2=q6(z2-z2f);
            case 8
                phi1=q8(z1-z1f);
                phi2=q8(z2-z2f);
            case 10
                phi1=q10(z1-z1f);
                phi2=q10(z2-z2f);
            case 12
                phi1=q12(z1-z1f);
                phi2=q12(z2-z2f);
            case 14
                phi1=q14(z1-z1f);
                phi2=q14(z2-z2f);
            case 16
                phi1=q16(z1-z1f);
                phi2=q16(z2-z2f);
        end
        for l=0:m-1
            for i=1:xa
                i2=mod(z2f(i)-l,2^(n-r));
                for k=0:m-1
                    ind=isp(n,r,mod(z1f(i)-k,2^r),i2);
                    a(ind)=a(ind)+phi1(k+1,i)*phi2(l+1,i)*y(i);
                end
            end
        end
    end
    
end


function [a]=abool_val_3d(s,m,x,n)

%    r = m/2;
%    dj=length(s);
%    k=0;
%    while((k+1)*2^(k)+2^(k-3)*k*(k-1)<dj)
%        k=k+1;
%     end
%     n=k;
%     if(dj~=(n+1)*2^(n)+2^(n-3)*n*(n-1))
%         disp('length of input vector must be like (n+1)*(n+2)*2^(n-1)');
%         return;
%     end
    [xa x12]=size(x);
    if(x12~=3)
        disp('list of points must be column-shaped [x1 y1 z1; x2 y2 z2; ...]');
        return;
    end
    a=zeros((n+1)*(n+2)*2^(n-1),1);
    phix=zeros(n+1,m);
    phiy=phix;
    phiz=phix;

    x=x-floor(x);   % now x is definitely in [0,1)
 


    for i=1:xa
        for j=0:n
            xx=x(i,:)*2^j;
            xxf=floor(xx);
            switch m
                case 2
                    phix(j+1,:)=q2(xx(1)-xxf(1));
                    phiy(j+1,:)=q2(xx(2)-xxf(2));
                    phiz(j+1,:)=q2(xx(3)-xxf(3));
                case 4
                    phix(j+1,:)=q4(xx(1)-xxf(1));
                    phiy(j+1,:)=q4(xx(2)-xxf(2));
                    phiz(j+1,:)=q4(xx(3)-xxf(3));
                case 6
                    phix(j+1,:)=q6(xx(1)-xxf(1));
                    phiy(j+1,:)=q6(xx(2)-xxf(2));
                    phiz(j+1,:)=q6(xx(3)-xxf(3));
                case 8
                    phix(j+1,:)=q8(xx(1)-xxf(1));
                    phiy(j+1,:)=q8(xx(2)-xxf(2));
                    phiz(j+1,:)=q8(xx(3)-xxf(3));
                case 10
                    phix(j+1,:)=q10(xx(1)-xxf(1));
                    phiy(j+1,:)=q10(xx(2)-xxf(2));
                    phiz(j+1,:)=q10(xx(3)-xxf(3));
                case 12
                    phix(j+1,:)=q12(xx(1)-xxf(1));
                    phiy(j+1,:)=q12(xx(2)-xxf(2));
                    phiz(j+1,:)=q12(xx(3)-xxf(3));
                case 14
                    phix(j+1,:)=q14(xx(1)-xxf(1));
                    phiy(j+1,:)=q14(xx(2)-xxf(2));
                    phiz(j+1,:)=q14(xx(3)-xxf(3));
                case 16
                    phix(j+1,:)=q16(xx(1)-xxf(1));
                    phiy(j+1,:)=q16(xx(2)-xxf(2));
                    phiz(j+1,:)=q16(xx(3)-xxf(3));
            end
        end
        
        for j=0:n
            for k3=0:m-1
%                u(k3+1)=0;
                k3t=mod(floor(2^(n-j)*x(i,3))-k3, 2^(n-j));
                for j1=0:j
                    for k2=0:m-1
%                        t(k2+1)=0;
                        k2t=mod(floor(2^(j-j1)*x(i,2))-k2, 2^(j-j1));
                        for k1=0:m-1
                            k1t=mod(floor(2^j1*x(i,1))-k1, 2^j1);
                            ind=isp3(n, j, j1, k1t, k2t, k3t);
                            a(ind)=a(ind)+phix(j1+1,k1+1)*phiy(j-j1+1,k2+1)*phiz(n-j+1,k3+1)*s(i);
                        end
                    end
                end
            end
        end
    end
end


function [ y ] = spline_val( a, m, x )
% Computes interpolated function values at any points x in 1d.
% If the x are only grid points, there would be a faster possibility however.
%
% [y] = spline_val(a,m,x)
%
% a     = spline coefficients
% m     = order of spline (4,6,8,10,12,14 or 16)

    r = m/2;
    if((r~=1) && (r~=2) && (r~=3) && (r~=4) && (r~=5) && (r~=6) && (r~=7) && (r~=8))
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    dj=length(a);
    n=round(log2(dj));
    if(dj~=2^n)
        disp('length of input vector must be power of 2');
        return;
    end
    [a1 a2]=size(a);
    if(a2==1)
        a=a.';
    end
    y=zeros(size(x));
    [x1 x2]=size(x);
    if(x2==1)
        x=x.';
    end
    
    x=x-floor(x);   % now x is definitely in [0,1)
    
    z=x*dj;
    z1=floor(z);
    switch m
        case 2
            phi=q2(z-z1);
        case 4
            phi=q4(z-z1);
        case 6
            phi=q6(z-z1);
        case 8
            phi=q8(z-z1);
        case 10
            phi=q10(z-z1);
        case 12
            phi=q12(z-z1);
        case 14
            phi=q14(z-z1);
        case 16
            phi=q16(z-z1);
    end
    for k=1:length(x)
        for i=0:m-1
            y(k)=y(k)+phi(i+1,k)*a(mod(z1(k)-i,dj)+1);
        end
    end
end

function [ y ] = aspline_val( a, m, x )
% Computes interpolated function values at any points x in 1d.
% If the x are only grid points, there would be a faster possibility however.
%
% [y] = spline_val(a,m,x)
%
% a     = spline coefficients
% m     = order of spline (4,6,8,10,12,14 or 16)

    r = m/2;
    if((r~=1) && (r~=2) && (r~=3) && (r~=4) && (r~=5) && (r~=6) && (r~=7) && (r~=8))
        disp('Order of spline must be 2, 4, 6, 8, 10, 12, 14 or 16');
        return;
    end
    dj=length(a);
    n=round(log2(dj));
    if(dj~=2^n)
        disp('length of input vector must be power of 2');
        return;
    end
    [a1 a2]=size(a);
    if(a2==1)
        a=a.';
    end
    y=zeros(size(x));
    [x1 x2]=size(x);
    if(x2==1)
        x=x.';
    end
    
    x=x-floor(x);   % now x is definitely in [0,1)
    
    z=x*dj;
    z1=floor(z);
    switch m
        case 2
            phi=q2(z-z1);
        case 4
            phi=q4(z-z1);
        case 6
            phi=q6(z-z1);
        case 8
            phi=q8(z-z1);
        case 10
            phi=q10(z-z1);
        case 12
            phi=q12(z-z1);
        case 14
            phi=q14(z-z1);
        case 16
            phi=q16(z-z1);
    end
    for k=1:length(x)
        for i=0:m-1
            y(mod(z1(k)-i,dj)+1)=y(mod(z1(k)-i,dj)+1)+phi(i+1,k)*a(k);
        end
    end
end



function ind=isp3(n, j, j1, k, l, m)
    ind=j*(j+1)*2^(n-1) + (j1*2^j + k*2^(j-j1) + l)*2^(n-j) + m+1;
end

function ind=isp(n,r,mu,l)
    ind=r*2^n+mu*2^(n-r)+l+1;
end

function y=q2(x)
    y=zeros(2,length(x));
    y(1,:)=x;
    x=x+1;
    y(2,:)=2-x;
end

function y=q4(x)
    y=zeros(4,length(x));
    y(1,:)=1/6*x.^3;
    x=x+1;
    y(2,:)=-1/2*x.^3 + 2*x.^2 - 2*x + 2/3;
    x=x+1;
    y(3,:)=1/2*x.^3 - 4*x.^2 + 10*x - 22/3;
    x=x+1;
    y(4,:)=-1/6*x.^3 + 2*x.^2 - 8*x + 32/3;
end

function y=q6(x)
    y=zeros(6,length(x));
    y(1,:)=1/120*x.^5;
    x=x+1;
    y(2,:)=-1/24*x.^5+1/4*x.^4-1/2*x.^3+1/2*x.^2-1/4*x+1/20;
    x=x+1;
    y(3,:)=1/12*x.^5-x.^4+9/2*x.^3-19/2*x.^2+39/4*x-79/20;
    x=x+1;
    y(4,:)=-1/12*x.^5+3/2*x.^4-21/2*x.^3+71/2*x.^2-231/4*x+731/20;
    x=x+1;
    y(5,:)=409/4*x+1/24*x.^5-1829/20+19/2*x.^3-89/2*x.^2-x.^4;
    x=x+1;
    y(6,:)=-54*x-1/120*x.^5+324/5-3*x.^3+18*x.^2+1/4*x.^4;
end

function y=q8(x)
    y=zeros(8,length(x));
    y(1,:)=1/5040*x.^7;
    x=x+1;
    y(2,:)=-1/720*x.^7+1/90*x.^6-1/30*x.^5+1/18*x.^4-1/18*x.^3+1/30*x.^2-1/90*x+1/630;
    x=x+1;
    y(3,:)=1/240*x.^7-1/15*x.^6+13/30*x.^5-3/2*x.^4+55/18*x.^3-37/10*x.^2+223/90*x-149/210;
    x=x+1;
    y(4,:)=-256/9*x.^3-5/3*x.^5+9*x.^4+1/6*x.^6-488/9*x-1/144*x.^7+53*x.^2+2477/105;
    x=x+1;
    y(5,:)=-64249/315+96*x.^3+3*x.^5-199/9*x.^4-2/9*x.^6+344*x+1/144*x.^7-737/3*x.^2;
    x=x+1;
    y(6,:)=-2647/18*x.^3-17/6*x.^5+53/2*x.^4+1/6*x.^6+139459/210-15683/18*x-1/240*x.^7+967/2*x.^2;
    x=x+1;
    y(7,:)=1889/18*x.^3+41/30*x.^5-31/2*x.^4-1/15*x.^6-187133/210+84881/90*x+1/720*x.^7-4237/10*x.^2;
    x=x+1;
    y(8,:)=-256/9*x.^3-4/15*x.^5+32/9*x.^4+1/90*x.^6-16384/45*x-1/5040*x.^7+131072/315+2048/15*x.^2;
end

function y=q10(x)
    y=zeros(10,length(x));
    y(1,:)=1/362880*x.^9;
    x=x+1;
    y(2,:)=-1/40320*x.^9+1/4032*x.^8-1/1008*x.^7+1/432*x.^6-1/288*x.^5+1/288*x.^4-1/432*x.^3+1/1008*x.^2-1/4032*x+1/36288;
    x=x+1;
    y(3,:)=1/10080*x.^9-1/504*x.^8+17/1008*x.^7-35/432*x.^6+71/288*x.^5-143/288*x.^4+287/432*x.^3-575/1008*x.^2+1151/4032*x-329/5184;
    x=x+1;
    y(4,:)=-1/4320*x.^9+233893/36288-8461/432*x.^3+2773/288*x.^4-901/288*x.^5-11083/576*x+3667/144*x.^2+289/432*x.^6-13/144*x.^7+1/144*x.^8;
    x=x+1;
    y(5,:)=1/2880*x.^9-5271131/36288+77555/432*x.^3-18731/288*x.^4+4475/288*x.^5+185525/576*x-45485/144*x.^2-1055/432*x.^6+35/144*x.^7-1/72*x.^8;
    x=x+1;
    y(6,:)=-1/2880*x.^9+43947619/36288-316195/432*x.^3+60019/288*x.^4-11275/288*x.^5-1220725/576*x+235765/144*x.^2+2095/432*x.^6-55/144*x.^7+5/288*x.^8;
    x=x+1;
    y(7,:)=1/4320*x.^9-167683997/36288+663581/432*x.^3-103277/288*x.^4+15941/288*x.^5+3818123/576*x-604043/144*x.^2-2441/432*x.^6+53/144*x.^7-1/72*x.^8;
    x=x+1;
    y(8,:)=-1/10080*x.^9-748207/432*x.^3+98407/288*x.^4-12871/288*x.^5-6064393/576*x+807745/144*x.^2+1675/432*x.^6-31/144*x.^7+316559287/36288+1/144*x.^8;
    x=x+1;
    y(9,:)=1/40320*x.^9+431441/432*x.^3-49049/288*x.^4+5561/288*x.^5+33046721/4032*x-3782969/1008*x.^2-629/432*x.^6+71/1008*x.^7-1/504*x.^8-287420489/36288;
    x=x+1;
    y(10,:)=-1/362880*x.^9+1562500/567-6250/27*x.^3+625/18*x.^4-125/36*x.^5-156250/63*x+62500/63*x.^2+25/108*x.^6-5/504*x.^7+1/4032*x.^8;
end

function y=q12(x)
    y=zeros(12,length(x));
    y(1,:)=1/39916800*x.^11;
    x=x+1;
    y(2,:)=1/3326400+(-1/302400+(1/60480+(-1/20160+(1/10080+(-1/7200+(1/7200+(-1/10080+(1/20160+(-1/60480+(1/302400-1/3628800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(3,:)=-1609/475200+(1877/100800+(-563/12096+(67/960+(-703/10080+(39/800+(-7/288+(29/3360+(-43/20160+(1/2880+(-1/30240+1/725760.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(4,:)=202277/207900+(-179489/50400+(8951/1512+(-19813/3360+(1231/315+(-723/400+(107/180+(-233/1680+(113/5040+(-1/420+(1/6720-1/241920.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(5,:)=-10611163/207900+(2343157/16800+(-261385/1512+(430747/3360+(-19889/315+(25991/1200+(-949/180+(509/560+(-547/5040+(43/5040+(-1/2520+1/120960.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(6,:)=1526438821/1663200+(-3585443/1800+(1692155/864+(-34519/30+(322843/720+(-12157/100+(16829/720+(-191/60+(433/1440+(-3/160+(1/1440-1/86400.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(7,:)=-2488249567/332640+(24128221/1800+(-46966553/4320+(157937/30+(-243361/144+(37739/100+(-215231/3600+(403/60+(-151/288+(13/480+(-1/1200+1/86400.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(8,:)=13202633671/415800+(-347376793/7200+(3581797/108+(-2177327/160+(666403/180+(-841271/1200+(1697/18+(-2161/240+(431/720+(-19/720+(1/1440-1/120960.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(9,:)=-31089216569/415800+(4950337489/50400+(-44133437/756+(69619493/3360+(-6148619/1260+(320323/400+(-8411/90+(13033/1680+(-2263/5040+(29/1680+(-1/2520+1/241920.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(10,:)=326605693613/3326400+(-11407451917/100800+(714406801/12096+(-17689187/960+(38498813/10080+(-441919/800+(81937/1440+(-14029/3360+(4313/20160+(-7/960+(1/6720-1/725760.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(11,:)=-31913472341/475200+(20777644249/302400+(-385593199/12096+(178527073/20160+(-16501187/10080+(1522729/7200+(-28063/1440+(12913/10080+(-1187/20160+(109/60480+(-1/30240+1/3628800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(12,:)=35831808/1925+(-2985984/175+(248832/35+(-62208/35+(10368/35+(-864/25+(72/25+(-6/35+(1/140+(-1/5040+(1/302400-1/39916800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
end

function y=q14(x)
    y=zeros(14,length(x));
    y(1,:)=1/6227020800*x.^13;
    x=x+1;
    y(2,:)=1/444787200+(-1/34214400+(1/5702400+(-1/1555200+(1/622080+(-1/345600+(1/259200+(-1/259200+(1/345600+(-1/622080+(1/1555200+(-1/5702400+(1/34214400-1/479001600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(3,:)=-17749/148262400+(26623/34214400+(-493/211200+(1331/311040+(-1109/207360+(1663/345600+(-277/86400+(83/51840+(-23/38400+(103/622080+(-17/518400+(1/228096+(-1/2851200+1/79833600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(4,:)=13799717/148262400+(-1253713/3110400+(15463/19200+(-1528619/1555200+(169477/207360+(-168923/345600+(18677/86400+(-18539/259200+(679/38400+(-2003/622080+(217/518400+(-19/518400+(1/518400-1/21772800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(5,:)=-190275385/17791488+(107798191/3110400+(-1073819/20736+(14688913/311040+(-3646973/124416+(4516901/345600+(-44617/10368+(10973/10368+(-13421/69120+(16301/622080+(-157/62208+(17/103680+(-1/155520+1/8709120.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(6,:)=1132024415/2965248+(-511004989/518400+(4052801/3456+(-44101327/51840+(2900689/6912+(-8557079/57600+(67043/1728+(-13067/1728+(12659/11520+(-12179/103680+(31/3456+(-1/2160+(1/69120-1/4838400.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(7,:)=-438619200697/74131200+(6563537603/518400+(-1077770407/86400+(1941159157/259200+(-105589099/34560+(51489193/57600+(-8331637/43200+(1341277/43200+(-214697/57600+(34153/103680+(-1799/86400+(77/86400+(-1/43200+1/3628800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(8,:)=1173329646263/24710400+(-2989797181/34560+(6266586067/86400+(-71110907/1920+(1331965789/103680+(-4089869/1280+(8438659/14400+(-46207/576+(471989/57600+(-1421/2304+(8617/259200+(-7/5760+(7/259200-1/3628800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(9,:)=-16133781407707/74131200+(178491341677/518400+(-21650701357/86400+(28786297763/259200+(-385146443/11520+(415741367/57600+(-49657207/43200+(5906123/43200+(-699467/57600+(82487/103680+(-3229/86400+(103/86400+(-1/43200+1/4838400.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(10,:)=17778941666987/29652480+(-866878641397/1036800+(3707327557/6912+(-21726158851/103680+(256320163/4608+(-1220410967/115200+(5146951/3456+(-540791/3456+(283147/23040+(-147767/207360+(205/6912+(-29/34560+(1/69120-1/8709120.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(11,:)=-89663174999039/88957440+(3899364075809/3110400+(-14878017329/20736+(77821523447/311040+(-7379355599/124416+(3488767099/345600+(-13159147/10368+(1237627/10368+(-580559/69120+(271699/622080+(-1015/62208+(43/103680+(-1/155520+1/21772800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(12,:)=149758213582337/148262400+(-3518739360077/3110400+(100807535467/172800+(-285265422391/1555200+(8136620657/207360+(-2084563807/345600+(59229257/86400+(-15119911/259200+(428177/115200+(-108967/622080+(3077/518400+(-71/518400+(1/518400-1/79833600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(13,:)=-82060398072319/148262400+(19248519952817/34214400+(-500968579687/1900800+(23439489013/311040+(-3042903439/207360+(710317217/345600+(-18406327/86400+(857797/51840+(-110959/115200+(25817/622080+(-667/518400+(31/1140480+(-1/2851200+1/479001600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(14,:)=110730297608/868725+(-7909306972/66825+(1129900996/22275+(-80707214/6075+(5764801/2430+(-823543/2700+(117649/4050+(-16807/8100+(2401/21600+(-343/77760+(49/388800+(-7/2851200+(1/34214400-1/6227020800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
end

function y=q16(x)
    y=zeros(16,length(x));
    y(1,:)=1/1307674368000*x.^15;
    x=x+1;
    y(2,:)=1/81729648000+(-1/5448643200+(1/778377600+(-1/179625600+(1/59875200+(-1/27216000+(1/16329600+(-1/12700800+(1/12700800+(-1/16329600+(1/27216000+(-1/59875200+(1/179625600+(-1/778377600+(1/5448643200-1/87178291200.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(3,:)=-245759/81729648000+(122879/5448643200+(-8777/111196800+(30719/179625600+(-15359/59875200+(1097/3888000+(-3839/16329600+(1919/12700800+(-137/1814400+(479/16329600+(-239/27216000+(17/8553600+(-59/179625600+(29/778377600+(-1/389188800+1/12454041600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(4,:)=250982993/40864824000+(-3216943/104781600+(306263/4276800+(-422039/4082400+(3092393/29937600+(-73537/972000+(342533/8164800+(-56929/3175200+(5399/907200+(-6259/4082400+(4133/13608000+(-97/2138400+(443/89812800+(-1/2721600+(1/59875200-1/2874009600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(5,:)=-60818083247/40864824000+(583985617/104781600+(-41636777/4276800+(472459651/44906400+(-235458647/29937600+(4186303/972000+(-14566907/8164800+(1806751/3175200+(-127721/907200+(110221/4082400+(-54107/13608000+(943/2138400+(-3197/89812800+(59/29937600+(-1/14968800+1/958003200.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(6,:)=8209662661631/81729648000+(-11439807787/38102400+(325348711/777600+(-5887322911/16329600+(1169014621/5443200+(-364114163/3888000+(504069311/16329600+(-99413621/12700800+(2791433/1814400+(-3824741/16329600+(744911/27216000+(-1873/777600+(2521/16329600+(-37/5443200+(1/5443200-1/435456000.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(7,:)=-227117922118657/81729648000+(262834766549/38102400+(-6204998297/777600+(93156273377/16329600+(-15338251427/5443200+(3959217421/3888000+(-4539817537/16329600+(741234187/12700800+(-17223991/1814400+(19526587/16329600+(-3146977/27216000+(6551/777600+(-7307/16329600+(89/5443200+(-1/2721600+1/261273600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(8,:)=263949046457549/6810804000+(-16293128113/198450+(5250119809/64800+(-2100331757/42525+(9432332239/453600+(-129654227/20250+(2026084289/1360800+(-8803642/33075+(5574587/151200+(-336419/85050+(739169/2268000+(-41/2025+(1249/1360800+(-13/453600+(1/1814400-1/203212800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(9,:)=-2094503395121971/6810804000+(112555890767/198450+(-220947199097/453600+(10985896723/42525+(-42912581681/453600+(3590811451/141750+(-6970697791/1360800+(26340038/33075+(-101552611/1058400+(761821/85050+(-1457311/2268000+(493/14175+(-1871/1360800+(17/453600+(-1/1587600+1/203212800.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(10,:)=40692706235403461/27243216000+(-30924410415847/12700800+(478950607261/259200+(-4713111843211/5443200+(508272631471/1814400+(-85886430863/1296000+(64452425381/5443200+(-6887943641/4233600+(104818643/604800+(-77903561/5443200+(8244101/9072000+(-11293/259200+(8311/5443200+(-67/1814400+(1/1814400-1/261273600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(11,:)=-378421881293789617/81729648000+(257226768752459/38102400+(-3563148178217/777600+(31360664470367/16329600+(-3025182105587/5443200+(457340707411/3888000+(-307142723857/16329600+(29386169077/12700800+(-400544071/1814400+(266789317/16329600+(-25317697/27216000+(31121/777600+(-20567/16329600+(149/5443200+(-1/2721600+1/435456000.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(12,:)=380983434478341553/40864824000+(-116937546567523/9525600+(2926068475973/388800+(-11632355401609/4082400+(2027867405243/2721600+(-138554713007/972000+(168288497893/8164800+(-7283451359/3175200+(179727799/907200+(-54211709/4082400+(9324613/13608000+(-5197/194400+(6233/8164800+(-41/2721600+(1/5443200-1/958003200.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(13,:)=-495290917576258127/40864824000+(1522258628957887/104781600+(-34684000126217/4276800+(125595697079581/44906400+(-19952059625207/29937600+(112984579153/972000+(-125174009627/8164800+(4944153121/3175200+(-111405641/907200+(30702211/4082400+(-4827707/13608000+(27073/2138400+(-29717/89812800+(179/29937600+(-1/14968800+1/2874009600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(14,:)=800924420340660241/81729648000+(-4511594214897307/419126400+(47122425359971/8553600+(-313050190968511/179625600+(22821494540881/59875200+(-237354142633/3888000+(120809458801/16329600+(-8773962751/12700800+(90931303/1814400+(-46129471/16329600+(3339841/27216000+(-34513/8553600+(17461/179625600+(-97/59875200+(1/59875200-1/12454041600.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(15,:)=-365836296342931439/81729648000+(24689326398020129/5448643200+(-237817155949817/111196800+(10195831986019/16329600+(-7550244231599/59875200+(72561559127/3888000+(-34148392079/16329600+(2294455169/12700800+(-22011737/1814400+(10342049/16329600+(-693839/27216000+(6647/8553600+(-3119/179625600+(19/70761600+(-1/389188800+1/87178291200.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
    x=x+1;
    y(16,:)=562949953421312/638512875+(-35184372088832/42567525+(2199023255552/6081075+(-137438953472/1403325+(8589934592/467775+(-536870912/212625+(33554432/127575+(-2097152/99225+(131072/99225+(-8192/127575+(512/212625+(-32/467775+(2/1403325+(-1/48648600+(1/5448643200-1/1307674368000.*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x).*x;
end

